import os
from qiutil.logging import logger
from . import reader

def edit(*in_files, **opts):
    """
    Edits the given DICOM files. The *dest* option can be either the
    destination directory path or an output file namer function.
    If *dest* is a directory path, then the output file location is
    a file in the given destination directory with the same unqualified
    file name as the input file. If the *dest* option is a function,
    then the output file location is the result of calling that
    function with the input file path as an argument. The default is
    to edit the file in-place.

    :param in_files: the input DICOM files or directories containing
        DICOM files
    :param opts: the following options:
    :keyword dest: the destination directory or file namer function
    :yield: the :meth`qidicom.reader.next` pydicom dicom object
    """
    # Prepare the destination directory.
    dest = opts.get('dest')
    if dest:
        if isinstance(dest, str):
            dest = os.path.abspath(dest)
            if not os.path.exists(dest):
                os.makedirs(dest)
    else:
        dest = os.getcwd()

    # Open the DICOM store on each DICOM file (skipping non-DICOM files),
    # yield to the edit callback and save to the file in the destination
    # directory.
    for ds in reader.iter_dicom(*in_files):
        logger(__name__).debug("Editing the DICOM file %s..." % ds.filename)
        # Delegate the edit.
        yield ds
        # Make the output file name.
        if isinstance(dest, str):
            # The dest option is a parent directory.
            _, fname = os.path.split(ds.filename)
            out_file = os.path.join(dest, fname)
        else:
            # The dest option is a function.
            out_file = dest(ds.filename)
        # Save the modified dataset.
        ds.save_as(out_file)
        logger(__name__).debug("Saved the edited DICOM file as %s." % out_file)



import os
import re
import operator
from dicom import datadict
from qiutil.logging import logger
from qiutil import functions
from . import (reader, writer)

# Uncomment the following lines to debug pydicom.
# import dicom
# dicom.debug(True)


def select(ds, *tags):
    """
    Reads the given DICOM dataset tags.
    
    :param ds: the pydicom dicom object
    :param tags: the names of tags to read (default all unbracketed tags)
    :return: the tag name => value dictionary
    """
    if not tags:
        # Skip tags with a bracketed name.
        tags = (de.name for de in ds if de.name and de.name[0] != '[')
    tdict = {}
    for t in tags:
        try:
            # The tag attribute.
            tattr = re.sub('\W', '', t)
            # Collect the tag value.
            tdict[t] = operator.attrgetter(tattr)(ds)
        except AttributeError:
            pass
    
    return tdict


class Editor(object):
    """DICOM tag editor."""
    
    def __init__(self, **edits):
        """
        Creates a new DICOM tag editor.

        :param edits: the :attr:`edits` tag value modifications
        """
        self.edits = edits
        """
        The DICOM header (I{name}, I{value}) tag modifications. The *value*
        can be a literal or a function. If it is a function, then the
        function is called on the current tag value, e.g.::

            meta.Editor(PatientID='Subject001',
                        BodyPartExamined=str.upper)
        """

        # The VR used to create a new tag entry is the first member of the
        # pydicom datadict value for the tag object.
        self._tvr = {name: datadict.get_entry(self._tag_for(name))[0]
                     for name in self.edits}
        """The edit {tag: DICOM VR} dictionary."""

    def edit(self, dataset):
        """
        Applies this editor's :attr:`edits` tag value modifications.

        :param dataset: the pydicom dicom dataset object
        """

        # Set the tag values.
        for attr, value in self.edits.iteritems():
            if functions.is_function(value):
                # Apply a function to the current tag value.
                if hasattr(dataset, attr):
                    resolved = value(getattr(dataset, attr))
                    setattr(dataset, attr, resolved)
            elif hasattr(dataset, attr):
                setattr(dataset, attr, value)
            else:
                tag = self._tag_for(attr)
                dataset.add_new(tag, self._tvr[attr], value)

    def _tag_for(self, name):
        return datadict.tag_for_name(name)
